#<profile4xi profile = "02" /> 
# -------------------------------------------------------------------------------------------------
#<program4xi program = "semlock4xi.py">
#<abstract4xi abstract = "provides SemLock4Xi class"/>
#<refer4xi reference = "synchronize.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'SemLock4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
try:                 # system ones
    import os
    import sys
    import signal
except:
    pass
try:                 # threading ones       
    import threading                
    import _thread
    import _winapi
       
except:
    pass
try:                 # most-common ones
    pass         
except:
    pass
try:                 # rare ones
    pass          
except:
    pass 
try:                 # imported-as ones 
    from time import time as _time, sleep as _sleep         
except:
    pass            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
try:                             # Binary ones
    import _multiprocessing
    from _multiprocessing import SemLock as _Semlock    
except:
    pass
try:                             # Python ones
    from multiprocessing.synchronize import SemLock 
except:
    print("semlock4xi.py: *** failed in from multiprocessing.synchronize import SemLock  ***") 
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from assert_spawning4xi import assert_spawning4xi
import constant_family4xi
import constant_heap4xi
import constant_pool4xi
import constant_synchronize4xi
from constant_system4xi import get_windows_system4xi
from constant_synchronize4xi import get_sem_value_synchronize4xi
from constant_synchronize4xi import get_sem_value_max_synchronize4xi
from constant_synchronize4xi import get_semaphore_synchronize4xi
from examine4xi import examine4xi
from print4xi import print4xi 
from popen4xi import Popen4Xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi
from time4xi import time4xi
#"
#/>




# =================================================================================================
#<class4xi class = "SemLock4Xi" base = "object" child = "None" 
#  synopsis = "semaphore locking"
#  usage = "
#   synchronize.py
#       BoundedSemaphore:
#           __init__:
#               SemLock.__init__(self, SEMAPHORE, value, value)
#
#   boundedsemaphore4xi.py:
#       SemLock4Xi.__init__(self, semaphore2xi, value2xi, maxvalue2xi)
#   lock4xi.py:
#       SemLock4Xi.__init__(self, semaphore2xi, value2xi, maxvalue2xi)
#   rlock4xi.py:
#       SemLock4Xi.__init__(self, recursive_semaphore2xi, value2xi, maxvalue2xi)
#"
#>
#<state4xi status = "NACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# =================================================================================================
class SemLock4Xi(object):
    '''
    semaphore locking                           
    '''
    
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = ('sense', 'reason', 'ace')          
    #"
    #/>
       
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-SemLock4Xi" usage = "
    #    initializes SemLock4Xi with (self, kind, value, maxvalue), which it gets from a user of this class">
    # usage in multiprocessing:
    #    1) SemLock.__init__(self, SEMAPHORE, 1, 1)       in synchronize.py        #kind = SEMAPHORE
    #    2) SemLock.__init__(self, RECURSIVE_MUTEX, 1, 1) in synchronize.py  #kind = RECURSIVE_MUTEX
    #
    # usage in dragonized multiprocessing4xi:
    # 
    #    1) lock4xi.py
    #           class Lock4Xi(SemLock4Xi):
    #               __init__(self) <--- does NOT take any parameters
    #                   SemLock4Xi.__init__(self, semaphore2xi, value2xi, maxvalue2xi) <--- does give SEMAPHORE as the second parameter    
    #
    #    2) rlock4xi.py
    #           class RLock4Xi(SemLock4Xi): 
    #               __init__(self):
    #                   SemLock4Xi.__init__(self, recursive_semaphore2xi, value2xi, maxvalue2xi) <--- does give RECURSIVE_MUTEX as the second parameter
    #
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, kind, value, maxvalue):                       #dragonized; kind type is recursive takes kindmaxvalue
        '''
        initializes SemLock4Xi  
        '''
        class2print = "SemLock4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/5b) started" + "; init-of-SemLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 04:37:06 04/15/13 Pacific Daylight Time                       
        
        message2print = "{(Step 1/5c) borning" + "; semlock2xi-of-SemLock4Xi-borning: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        # 04:37:06 04/15/13 Pacific Daylight Time
        
        message2print = "{(Step 1/5d) inheriting" + "; object-inheriting-by-semlock2xi-of-SemLock4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'object'>                        
                
        message2print = "{(Step 1/5e) passed" + "; kind-passed-to-init-of-SemLock4Xi: " + str(kind) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        # 0          
                
        message2print = "{(Step 1/5f) passed" + "; value-passed-to-init-of-SemLock4Xi: " + str(value) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        # 1                
                
        message2print = "{(Step 1/5g) passed" + "; maxvalue-passed-to-init-of-SemLock4Xi: " + str(maxvalue) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        # 2147483647           
        

        if examine4xi(class2print, method2print): 
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(StepOriginal) stepping" + "; (StepOriginal)-stepping-by-init-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
                 
            message2print = "{(StepOriginal) passing" + "; semlock2xi-passing-to-Semlock-init-by-init-of-SemLock4Xi: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(StepOriginal) passing" + "; kind-passing-to-Semlock-init-by-init-of-SemLock4Xi: " + str(kind) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
            
            message2print = "{(StepOriginal) passing" + "; value-passing-to-Semlock-init-by-init-of-SemLock4Xi: " + str(value) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
            message2print = "{(StepOriginal) passing" + "; maxvalue-passing-to-Semlock-init-by-init-of-SemLock4Xi: " + str(maxvalue) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                         
            
            Semlock.__init__(self, kind, value, maxvalue)
            
            message2print = "{performed" + "; Semlock-init-performed-by-init-of-SemLock4Xi: " + str(Semlock.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
        else:
            # ---------------------------------------------------------------------------------------------
            # note: the caller is responsible to provide the right parameters
            #       integer required for kind parameter which is SEMAPHORE or RECURSIVE_MUTEX list(range(2))
            # ---------------------------------------------------------------------------------------------
                         
            message2print = "{(Step 2/5b) passing" + "; kind-passing-to-SemLock-by-init-of-SemLock4Xi: " + str(kind) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 0                 
                            
            message2print = "{(Step 2/5c) passing" + "; value-passing-to-SemLock-by-init-of-SemLock4Xi: " + str(value) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 1                                   
                            
            message2print = "{(Step 2/5d) passing" + "; maxvalue-passing-to-SemLock-by-init-of-SemLock4Xi: " + str(maxvalue) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 2147483647
            
            message2print = "{(Step 2/5e) calling" + "; SemLock-of-multiprocessing-calling-by-init-of-SemLock4Xi: " + str(_multiprocessing.SemLock) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)       
            # <class '_multiprocessing.SemLock'>                

            semaphorelock2xi = self._semlock = _multiprocessing.SemLock(kind, value, maxvalue) 

            
            message2print = "{(Step 2/5f) set" + "; semaphorelock2xi-set-by-init-of-SemLock4Xi to: " + str(semaphorelock2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <_multiprocessing.SemLock object at 0x00000000039CAF08> 
            
            message2print = "{(Step 2/5g) set" + "; semlock-of-semlock2xi-set-by-init-of-SemLock4Xi to: " + str(self._semlock)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <_multiprocessing.SemLock object at 0x00000000039CAF08>
            
            self._make_methods()
            
            message2print = "{(Step 3/5b) performed" + "; make-methods-of-semlock2xi-performed-by-init-of-SemLock4Xi: " + str(self._make_methods) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <bound method RLock4Xi._make_methods of <RLock4Xi(None, 0)>>
            
            if sys.platform != get_windows_system4xi(): #dragonized

                # -------------------------------------------------------------------------------------------------------------------------------------------
                # message2print = "{(Step 4/5a) stepping" + "; (Step 4/5a)-stepping-by-init-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
                # print4xi(class2print, method2print, message2print)
                # -------------------------------------------------------------------------------------------------------------------------------------------
    
                # -----------------------------------------------------------------------------------------  
                #<function4xi function = "after-fork4xi-of-SemLock4Xi" usage = "
                #    embedded function of SemLock4Xi
                #"> 
                #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
                # -----------------------------------------------------------------------------------------  
                
                def _after_fork4xi(obj):
                    
                    message2print = "{(Step 4/5b) passed" + "; object-passed-to-after-fork4xi-of-SemLock4Xi: " + str(obj) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)  
                    
                    obj._semlock._after_fork()
                    
                    message2print = "{(Step 4/5c) performed" + "; object-semlock-after-fork-performed-by-init-of-SemLock4Xi: " + str(obj._semlock._after_fork) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)             
                #</function4xi>
                
                # -----------------------------------------------------------------------------------------  
                #                                     
                # ----------------------------------------------------------------------------------------- 
                                
                message2print = "{(Step 4/5d) passing" + "; after-fork4xi-passing-to-register-after-fork4xi-by-init-of-SemLock4Xi: " + str(_after_fork4xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                                
                                
                register_after_fork4xi(self, _after_fork4xi)
                
                message2print = "{(Step 4/5e) performed" + "; register-after-fork4xi-performed-by-init-of-SemLock4Xi: " + str(register_after_fork4xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                                               
            

            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 5/5a) stepping" + "; (Step 5/5a)-stepping-by-init-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
            
            message2print = "{(Step 5/5b) inherited" + "; object-inherited-by-semlock2xi-of-SemLock4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)         
            # <class 'object'>      
            
            message2print = "{(Step 5/5c) activated" + "; semlock2xi-of-SemLock4Xi-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 04:54:08 04/15/13 Pacific Daylight Time                 
            message2print = "{(Step 5/5d) returning" + "; init-of-SemLock4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 04:54:08 04/15/13 Pacific Daylight Time
            
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "make-methods-of-SemLock4Xi" usage = "makes methods with (self)">                       
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _make_methods(self):
        '''
        makes methods
        '''
        class2print = "SemLock4Xi"
        method2print = "_make_methods" 
        
        # -------------------------------------------------------------------------------------------------------------------------------------------
        # message2print = "{(Step 1/6a) stepping" + "; (Step 1/6a)-stepping-by-make-methods-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
        # print4xi(class2print, method2print, message2print)
        # -------------------------------------------------------------------------------------------------------------------------------------------
        
        message2print = "{(Step 1/6b) started" + "; make-methods-of-SemLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 04:37:06 04/15/13 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        ack2original = False
        message2print = "{(Step 1/6c) commented" + "; make-methods-of-SemLock4Xi-commented: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 04:37:06 04/15/13 Pacific Daylight Time
        try:
            message2print = "{(Step 1/6d) commented" + "; make-methods-of-SemLock4Xi-commented: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 04:37:06 04/15/13 Pacific Daylight Time
            if examine4xi(class2print, method2print):
               ack2original = True
               message2print = "{(Step 1/6e) commented" + "; make-methods-of-SemLock4Xi-commented: " + time4xi() + "}"
               print4xi(class2print, method2print, message2print)   
               # 01:40:34 04/08/13 Pacific Daylight Time
                          
        except:
            message2print = "{(Step 1/6f) commented" + "; make-methods-of-SemLock4Xi-commented: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            pass
        message2print = "{(Step 1/6g) commented" + "; make-methods-of-SemLock4Xi-commented: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 04:37:06 04/15/13 Pacific Daylight Time
        if ack2original is True:
            message2print = "{(Step 1/6h) commented" + "; make-methods-of-SemLock4Xi-commented: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 01:40:34 04/08/13 Pacific Daylight Time

            #            
            try: 
                message2print = "{(Step 1/6i) commented" + "; make-methods-of-SemLock4Xi-commented: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 01:43:28 04/08/13 Pacific Daylight Time}    
                Semlock._make_methods(self)
                    
                message2print = "{(Step 1/6j) commented" + "; make-methods-of-SemLock4Xi-commented: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                #   
            except:
                message2print = "{(Step 1/6z) commented" + "; make-methods-of-SemLock4Xi-commented: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                # 01:43:28 04/08/13 Pacific Daylight Time} <-- JATKA - why does it fail here ???   <-- is it necessary ?                            
        else:
            message2print = "{(Step 1/6k) commented" + "; make-methods-of-SemLock4Xi-commented: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 04:37:06 04/15/13 Pacific Daylight Time          
             
            message2print = "{(Step 2/6b) using" + "; semlock-acquire-of-semlock2xi-using-by-make-methods-of-SemLock4Xi: " + str(self._semlock.acquire)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            self.acquire = self._semlock.acquire

            message2print = "{(Step 2/6c) set" + "; acquire-of-semlock2xi-set-by-make-methods-of-SemLock4Xi to: " + str(self.acquire)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <built-in method acquire of _multiprocessing.SemLock object at 0x00000000039CAF08>
            
            message2print = "{(Step 3/6b) using" + "; semlock-release-of-semlock2xi-using-by-make-methods-of-SemLock4Xi: " + str(self._semlock.release)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <built-in method release of _multiprocessing.SemLock object at 0x00000000039CAF08>               
            
            self.release = self._semlock.release
            
            message2print = "{(Step 3/6c) set" + "; release-of-semlock2xi-set-by-make-methods-of-SemLock4Xi to: " + str(self.release)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <built-in method release of _multiprocessing.SemLock object at 0x00000000039CAF08>              

            message2print = "{(Step 4/6b) using" + "; semlock-enter-of-semlock2xi-using-by-make-methods-of-SemLock4Xi: " + str(self._semlock.__enter__)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
                            
            
            self.__enter__ = self._semlock.__enter__
            
            message2print = "{(Step 4/6c) set" + "; enter-of-semlock2xi-set-by-make-methods-of-SemLock4Xi to: " + str(self.__enter__)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <built-in method __enter__ of _multiprocessing.SemLock object at 0x00000000039CAF08>
            
            message2print = "{(Step 5/6b) using" + "; semlock-exit-of-semlock2xi-using-by-make-methods-of-SemLock4Xi: " + str(self._semlock.__exit__)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <built-in method __exit__ of _multiprocessing.SemLock object at 0x00000000039CAF08>                
            
            self.__exit__ = self._semlock.__exit__
              
            message2print = "{(Step 5/6c) set" + "; exit-of-semlock2xi-set-by-make-methods-of-SemLock4Xi to: " + str(self.__exit__)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <built-in method __exit__ of _multiprocessing.SemLock object at 0x00000000039CAF08>
            
            message2print = "{(Step 6/6b) returning" + "; make-methods-of-SemLock4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 04:37:06 04/15/13 Pacific Daylight Time

    #</methodize4xi>
                        
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "getstate-of-SemLock4Xi" usage = "gets state with (self)">                            
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __getstate__(self):
        '''
        SemLock4Xi method __getstate__:  gets state with (self)
        '''
     
        class2print = "SemLock4Xi"
        method2print = "__getstate__"
        
        # -------------------------------------------------------------------------------------------------------------------------------------------
        # message2print = "{(Step 1/3a) stepping" + "; (Step 1/3a)-stepping-by-make-methods-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
        # print4xi(class2print, method2print, message2print)
        # -------------------------------------------------------------------------------------------------------------------------------------------
      
        message2print = "{(Step 1/3b) started" + "; getstate-of-SemLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
         

        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        if examine4xi(class2print, method2print):
        
        
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(StepOriginal) stepping" + "; (StepOriginal)-stepping-by-make-methods-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
              
            message2print = "{(StepOriginal) returning" + "; Semlock-getstate-function-returning-by-getstate-of-SemLock4Xi: " + str(Semlock.__getstate__) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                            
                   
            return Semlock.__getstate__(self)
        else:    
                
        
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 2/3a) stepping" + "; (Step 2/3a)-stepping-by-make-methods-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
      
            assert_spawning4xi(self)
            
            message2print = "{(Step 2/3b) performed" + "; assert-spawning4xi-performed-by-getstate-of-SemLock4Xi: " + str(assert_spawning4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                                        
            
            semaphorelock2xi = self._semlock
            
            message2print = "{(Step 2/3c) set" + "; semaphore-lock2xi-set-by-getstate-of-SemLock4Xi to: " + str(sl4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 3/3a) stepping" + "; (Step 3/3a)-stepping-by-make-methods-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
                    
            message2print = "{(Step 3/3b) passing" + "; semaphore-handle-passing-to-Popen4Xi-duplicate-for-child-by-getstate-of-SemLock4Xi: " + str(semaphorelock2xi.handle) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                             
            message2print = "{(Step 3/3c) passing" + "; semaphore-kind-passing-to-Popen4Xi-duplicate-for-child-by-getstate-of-SemLock4Xi: " + str(semaphorelock2xi.kind) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                                 
            message2print = "{(Step 3/3d) passing" + "; semaphore-maxvalue-passing-to-Popen4Xi-duplicate-for-child-by-getstate-of-SemLock4Xi: " + str(semaphorelock2xi.maxvalue) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            
            message2print = "{(Step 3/3e) returning" + "; Popen4Xi-duplicate-for-child-returning-by-getstate-of-SemLock4Xi: " + str(Popen4Xi.duplicate_for_child) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                            
              
            return (Popen4Xi.duplicate_for_child(semaphorelock2xi.handle), semaphorelock2xi.kind, semaphorelock2xi.maxvalue)
            

    #</methodize4xi>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "setstate-of-SemLock4Xi" usage = "sets state with (self, state">                        
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __setstate__(self, state):
        '''
        SemLock4Xi method __setstate__: sets state with (self, state)
        '''    
        class2print = "SemLock4Xi"
        method2print = "__setstate__"
        
        # -------------------------------------------------------------------------------------------------------------------------------------------
        # message2print = "{(Step 1/4a) stepping" + "; (Step 1/4a)-stepping-by-setstate-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
        # print4xi(class2print, method2print, message2print)
        # -------------------------------------------------------------------------------------------------------------------------------------------
        
        message2print = "{(Step 1/4b) started" + "; setstate-of-SemLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        message2print = "{(Step 1/4c) passed" + "; state-passed-to-setstate-of-SemLock4Xi: " + str(state) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------    
        #
        # -----------------------------------------------------------------------------------------                 
        if examine4xi(class2print, method2print):
        
        
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(StepOriginal) stepping" + "; (StepOriginal)-stepping-by-setstate-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
                   
            message2print = "{(StepOriginal) passing" + "; state-passing-to-Semlock-setstate-by-setstate-of-SemLock4Xi: " + str(state) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
              
            Semlock.__setstate__(self, state)
            
            message2print = "{(StepOriginal) performed" + "; Semlock-setstate-performed-by-setstate-of-SemLock4Xi: " + str(Semlock.__setstate__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                            
        else:                   
        
        
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 2/4a) stepping" + "; (Step 2/4a)-stepping-by-setstate-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
                        

            self._semlock = _multiprocessing.SemLock._rebuild(*state)
            
            message2print = "{(Step 2/4b) set" + "; semlock-of-semlock2xi-set-by-setstate-of-SemLock4Xi to: " + str(self._semlock)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    

            # recreated blocker with handle 
            
        
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 3/4a) stepping" + "; (Step 3/4a)-stepping-by-setstate-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
                    
            
            self._make_methods()
            
            message2print = "{(Step 3/4b) performed" + "; make-methods-of-semlock2xi-performed-by-setstate-of-SemLock4Xi: " + str(self._make_methods) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                                    
        
            # -------------------------------------------------------------------------------------------------------------------------------------------
            # message2print = "{(Step 4/4a) stepping" + "; (Step 4/4a)-stepping-by-setstate-of-SemLock4Xi: " + "~~"  + "; time: " + time4xi() + "}"
            # print4xi(class2print, method2print, message2print)
            # -------------------------------------------------------------------------------------------------------------------------------------------
                
            message2print = "{(Step 4/4b) returning" + "; setstate-of-SemLock4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
    #</methodize4xi>
                          
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "setstate-of-SemLock4Xi" usage = "returns semlock enter">                        
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __enter__(self):
        '''
        SemLock4Xi method __enter__: returns semlock enter
        '''    
        class2print = "SemLock4Xi"
        method2print = "__enter__"
        
        # -------------------------------------------------------------------------------------------------------------------------------------------
        # (Step 1/2) 
        # -------------------------------------------------------------------------------------------------------------------------------------------
        message2print = "{(Step 1/2) started" + "; enter-of-SemLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        # -------------------------------------------------------------------------------------------------------------------------------------------
        # (Step 2/2) 
        # -------------------------------------------------------------------------------------------------------------------------------------------
        message2print = "{(Step 2/2) returning" + "; enter-of-SemLock4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                                
        return self._semlock.__enter__()                   
                                    
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "setstate-of-SemLock4Xi" usage = "returns semlock exit">                        
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __exit__(self):
        '''
        SemLock4Xi method __exit__: returns semlock exit
        '''    
        class2print = "SemLock4Xi"
        method2print = "__exit__"
        
        # -------------------------------------------------------------------------------------------------------------------------------------------
        # (Step 1/2) 
        # -------------------------------------------------------------------------------------------------------------------------------------------
        message2print = "{(Step 1/2) started" + "; exit-of-SemLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        # -------------------------------------------------------------------------------------------------------------------------------------------
        # (Step 2/2) 
        # -------------------------------------------------------------------------------------------------------------------------------------------
        message2print = "{(Step 2/2) returning" + "; exit-of-SemLock4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                                
        return self._semlock.__exit__(*args)       
                                    
    #</methodize4xi>    
                                           
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-SemLock4Xi" usage = "senses SemLock4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self): 
        '''
        senses SemLock4Xi class objects
        '''
 
        class2print = 'SemLock4Xi'
        method2print = 'sense'
        

        message2print = "{started" + "; sense-of-SemLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; SemLock4Xi-sensing-by-sense-of-SemLock4Xi: " + str(SemLock4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                       
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-SemLock4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['handle-of-semlock2xi'] = str(self._semlock)     
            kb2xi['acquire-of-semlock2xi'] = str(self._semlock.acquire)         
            kb2xi['release-of-semlock2xi'] = str(self._semlock.release)     
            kb2xi['enter-of-semlock2xi'] = str(self._semlock.__enter__)          
            kb2xi['exit-of-semlock2xi'] = str(self._semlock.__exit__)     
            kb2xi['make-methods-of-semlock2xi'] = str(self._make_methods)        
        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-SemLock4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
                      
        message2print = "{sensed" + "; SemLock4Xi-sensed-by-sense-of-SemLock4Xi: " + str(SemLock4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)      
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-SemLock4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-SemLock4Xi" usage = "reasons SemLock4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        reasons SemLock4Xi class objects
        '''
       
        
        class2print = 'SemLock4Xi'
        method2print = 'reason'


        message2print = "{started" + "; reason-of-SemLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; SemLock4Xi-reasoning-by-reason-of-SemLock4Xi: " + str(SemLock4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
                
                
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{got" + "; kb4xi-got-by-reason-of-SemLock4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['semlock-of-semlock2xi'] = str(self._semlock)     
            kb4xi['acquire-of-semlock2xi'] = str(self.acquire)         
            kb4xi['release-of-semlock2xi'] = str(self.release)     
            kb4xi['enter-of-semlock2xi'] = str(self.__enter__)          
            kb4xi['exit-of-semlock2xi'] = str(self.__exit__)     
            kb4xi['make-methods-of-semlock2xi'] = str(self._make_methods)              
        except:
            message2print = "{yielded" + "; kb4xi-yielded-by-reason-of-SemLock4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
    
          
        
        message2print = "{reasoned" + "; SemLock4Xi-reasoned-by-reason-of-SemLock4Xi: " + str(SemLock4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-SemLock4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                 
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-SemLock4Xi" usage = "aces SemLock4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        aces SemLock4Xi class objects 
        '''
         
        
        
        class2print = 'SemLock4Xi'
        method2print = 'ace'
        

        message2print = "{started" + "; ace-of-SemLock4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
              
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; SemLock4Xi-acing-by-ace-of-SemLock4Xi: " + str(SemLock4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                     
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-SemLock4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        
          
        message2print = "{aced" + "; SemLock4Xi-aced-by-ace-of-SemLock4Xi: " + str(SemLock4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-SemLock4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                                
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_proxy_mp_test4xi.xml
#   sense-proxy-mp-test4xi.xml
#   reason-proxy-mp-test4xi.xml
#   ace-proxy-mp-test4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of semlock4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>
